@@ -1,3 +1,5 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* Metacity Session Management */
 
 /* 
@@ -1790,7 +1792,7 @@ warn_about_lame_clients_and_finish_interact (gboolean shutdown)
   GError *err;
   GIOChannel *channel;
   LameClientsDialogData *d;
-  Time timestamp;
+  guint32 timestamp;
   char timestampbuf[32];
   
   lame = NULL;
@@ -1834,28 +1836,8 @@ warn_about_lame_clients_and_finish_interact (gboolean shutdown)
   
   lame = g_slist_sort (lame, (GCompareFunc) windows_cmp_by_title);
 
-  
-    {
-      XEvent property_event;
-      MetaDisplay *display;
-
-      display = meta_displays_list ()->data;
-
-      /* Using the property XA_PRIMARY because it's safe; nothing
-       * would use it as a property. The type doesn't matter.
-       */
-      XChangeProperty (display->xdisplay,
-                       display->leader_window,
-                       XA_PRIMARY, XA_STRING, 8,
-                       PropModeAppend, NULL, 0);
-      XWindowEvent (display->xdisplay,
-                    display->leader_window,
-                    PropertyChangeMask,
-                    &property_event);
-
-      timestamp = property_event.xproperty.time;
-    }
-  sprintf (timestampbuf, "%lu", timestamp);
+  timestamp = meta_display_get_current_time_roundtrip (displays->data);  
+  sprintf (timestampbuf, "%u", timestamp);
 
   len = g_slist_length (lame);
   len *= 2; /* titles and also classes */
